package com.totopia.server.modules.dashboard.controller;

import com.totopia.server.commons.exception.ResourceNotFoundException;
import com.totopia.server.modules.dashboard.entity.DashboardEntity;
import com.totopia.server.modules.dashboard.repository.DashboardRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.transaction.Transactional;
import java.util.List;

@RestController
public class DashboardController {
  @Autowired
  private DashboardRepository dashboardRepository;

  @PostMapping(value = "/dashboards")
  @ResponseStatus(code = HttpStatus.CREATED)
  public DashboardEntity save(@RequestBody DashboardEntity dashboard) {
    return dashboardRepository.save(dashboard);
  }

  @PreAuthorize("hasAnyRole('ROLE_USER', 'ROLE_ADMIN', 'ROLE_SUPER_ADMIN')")
  @GetMapping(value = "/dashboards")
  public List<DashboardEntity> all() {
    return dashboardRepository.findByOrderBySortOrder();
  }

  @GetMapping(value = "/dashboards/{id}")
  public DashboardEntity findById(@PathVariable Integer id) {
    return dashboardRepository.findById(id).orElseThrow(() -> new ResourceNotFoundException("Dashboard", "id", id));
  }

  @DeleteMapping(value = "/dashboards/{id}")
  @Transactional
  public ResponseEntity<?> deleteDashboard(@PathVariable Integer id) {

    return dashboardRepository.findById(id).map(dashboard -> {
      dashboardRepository.decreaseSortOrder(dashboard.getSortOrder(), (int) dashboardRepository.count());
      dashboardRepository.delete(dashboard);

      return ResponseEntity.ok().build();
    }).orElseThrow(() -> new ResourceNotFoundException("Dashboard", "id", id));
  }

  @PutMapping(value = "/dashboards/{id}")
  public ResponseEntity<DashboardEntity> updateDashboard(@PathVariable Integer id,
      @RequestBody DashboardEntity newDashboard) {

    return dashboardRepository.findById(id).map(dashboard -> {

      dashboardRepository.save(dashboard);
      return ResponseEntity.ok(dashboard);
    }).orElseThrow(() -> new ResourceNotFoundException("Dashboard", "id", id));
  }

  @PutMapping(value = "/dashboards/{id}/display/{display}")
  public ResponseEntity<DashboardEntity> updateDashboardDisplay(@PathVariable Integer id,
      @PathVariable Boolean display) {

    return dashboardRepository.findById(id).map(dashboard -> {
      dashboard.setDisplay(display);

      dashboardRepository.save(dashboard);
      return ResponseEntity.ok(dashboard);
    }).orElseThrow(() -> new ResourceNotFoundException("Dashboard", "id", id));

  }

  @PutMapping(value = "/dashboards/{id}/sort_order/{targetSortOrder}")
  @Transactional
  public ResponseEntity<DashboardEntity> updateDashboardSortOrder(@PathVariable Integer id,
      @PathVariable Integer targetSortOrder) {

    return dashboardRepository.findById(id).map(dashboard -> {
      if (targetSortOrder.equals(dashboard.getSortOrder())) {
        return ResponseEntity.ok(dashboard);
      }

      if (0 > targetSortOrder || dashboardRepository.count() < targetSortOrder) {
        return ResponseEntity.ok(dashboard);
      }

      Integer sourceSortOrder = dashboard.getSortOrder();

      if (targetSortOrder > sourceSortOrder) {
        dashboardRepository.decreaseSortOrder(sourceSortOrder, targetSortOrder);
      } else {
        dashboardRepository.increaseSortOrder(sourceSortOrder, targetSortOrder);
      }

      dashboard.setSortOrder(targetSortOrder);
      dashboardRepository.save(dashboard);

      return ResponseEntity.ok(dashboard);
    }).orElseThrow(() -> new ResourceNotFoundException("Dashboard", "id", id));

  }
}
